@@ -11,36 +11,54 @@
 
 
 import os
-import sys
 import platform
+import subprocess
+import sys
+import unittest
 import warnings
 from contextlib import contextmanager
-import subprocess
+from unittest.mock import MagicMock
 
-import numpy as np
-from numpy import (
-    array, arange, empty, zeros, int32, int64, uint16, cdouble, float64, rec,
-    copy, ones_like, where, all as alltrue, linspace,
-    sum, prod, sqrt, fmod, floor, ceil,
-    sin, cos, tan, arcsin, arccos, arctan, arctan2,
-    sinh, cosh, tanh, arcsinh, arccosh, arctanh,
-    log, log1p, log10, exp, expm1, conj)
 import numpy
-from numpy.testing import (assert_equal, assert_array_equal,
-                           assert_array_almost_equal, assert_allclose)
-from numpy import shape, allclose, array_equal, ravel, isnan, isinf
+import numpy as np
+from numpy import all as alltrue
+from numpy import (allclose, arange, arccos, arccosh, arcsin, arcsinh, arctan,
+                   arctan2, arctanh, array, array_equal, cdouble, ceil, conj,
+                   copy, copysign, cos, cosh, empty, exp, expm1, float64,
+                   floor, fmod, hypot, int32, int64, isfinite, isinf, isnan,
+                   linspace, log, log1p, log2, log10, maximum, minimum,
+                   nextafter, ones_like, prod, ravel, rec, round, shape, sign,
+                   signbit, sin, sinh, sqrt, sum, tan, tanh, trunc, uint16,
+                   where, zeros)
+from numpy.testing import (assert_allclose, assert_array_almost_equal,
+                           assert_array_equal, assert_equal)
 
 import numexpr
-from numexpr import E, NumExpr, evaluate, re_evaluate, validate, disassemble, use_vml
+from numexpr import (E, NumExpr, disassemble, evaluate, re_evaluate, use_vml,
+                     validate)
 from numexpr.expressions import ConstantNode
 from numexpr.utils import detect_number_of_cores
 
-import unittest
+try:
+    import pytest
+    pytest_available = True
+except ImportError:
+    pytest_available = False
 
 TestCase = unittest.TestCase
 
 double = np.double
 long = int
+MAX_THREADS = 16
+
+
+if not pytest_available:
+    def identity(f):
+        return f
+
+    pytest = MagicMock()
+    pytest.mark = MagicMock()
+    pytest.mark.thread_unsafe = identity
 
 
 class test_numexpr(TestCase):
@@ -308,15 +326,18 @@ class test_numexpr(TestCase):
         res = evaluate('where(a, b, c)')
         assert_array_equal(res, c)
 
+    # Comment out this test completely, as modern Python optimizes handling refcounts.
+    # See #511 for more info.
     @unittest.skipIf(hasattr(sys, "pypy_version_info"),
                      "PyPy does not have sys.getrefcount()")
-    def test_refcount(self):
+    def _test_refcount(self):
         # Regression test for issue #310
         a = array([1])
         assert sys.getrefcount(a) == 2
         evaluate('1')
         assert sys.getrefcount(a) == 2
 
+    @pytest.mark.thread_unsafe
     def test_locals_clears_globals(self):
         # Check for issue #313, whereby clearing f_locals also clear f_globals
         # if in the top-frame. This cannot be done inside `unittest` as it is always
@@ -340,6 +361,7 @@ class test_numexpr(TestCase):
 
 
 
+@pytest.mark.thread_unsafe
 class test_numexpr2(test_numexpr):
     """Testing with 2 threads"""
     nthreads = 2
@@ -436,6 +458,47 @@ class test_evaluate(TestCase):
         else:
             raise ValueError("should raise exception!")
 
+        x = np.ones(10, dtype='bool')
+        y = np.zeros(10, dtype='bool')
+        assert_array_equal(evaluate("x & y"), x & y) # and
+        assert_array_equal(evaluate("x ^ y"), x ^ y) # xor
+        assert_array_equal(evaluate("x | y"), x | y) # or
+        assert_array_equal(evaluate("~x"), ~x) # invert
+
+    def test_bitwise_operators(self):
+        x = arange(10, dtype='i4')
+        y = arange(10, dtype='i4')
+        assert_array_equal(evaluate("x & y"), x & y) # and
+        assert_array_equal(evaluate("x ^ y"), x ^ y) # xor
+        assert_array_equal(evaluate("x | y"), x | y) # or
+        assert_array_equal(evaluate("~x"), ~x) # invert
+
+        x = arange(10, dtype='i8')
+        y = arange(10, dtype='i8')
+        assert_array_equal(evaluate("x & y"), x & y) # and
+        assert_array_equal(evaluate("x ^ y"), x ^ y) # xor
+        assert_array_equal(evaluate("x | y"), x | y) # or
+        assert_array_equal(evaluate("~x"), ~x) # invert
+
+    def test_maximum_minimum(self):
+        for dtype in [float, double, int, np.int64]:
+            x = arange(10, dtype=dtype)
+            y = 2 * arange(10, dtype=dtype)[::-1]
+            assert_array_equal(evaluate("maximum(x,y)"), maximum(x,y))
+            assert_array_equal(evaluate("minimum(x,y)"), minimum(x,y))
+
+    def test_sign_round(self):
+        for dtype in [float, double, np.int32, np.int64, complex]:
+            x = arange(10, dtype=dtype)
+            y = 2 * arange(10, dtype=dtype)[::-1]
+            r = x-y
+            if not np.issubdtype(dtype, np.integer):
+                r[-1] = np.nan
+            assert evaluate("round(r)").dtype == round(r).dtype
+            assert evaluate("sign(r)").dtype == sign(r).dtype
+            assert_array_equal(evaluate("sign(r)"), sign(r))
+            assert_array_equal(evaluate("round(r)"), round(r))
+
     def test_rational_expr(self):
         a = arange(1e6)
         b = arange(1e6) * 0.1
@@ -511,6 +574,7 @@ class test_evaluate(TestCase):
         else:
             self.fail()
 
+    @pytest.mark.thread_unsafe
     def test_sanitize(self):
         with _environment('NUMEXPR_SANITIZE', '1'):
             # Forbid dunder
@@ -582,11 +646,14 @@ class test_evaluate(TestCase):
             evaluate('1.5j')
             evaluate('3.j')
 
+            #pass imaginary with scientific notation
+            evaluate('1.2e3+4.5e6j')
+
             # pass forbidden characters within quotes
             x = np.array(['a', 'b'], dtype=bytes)
             evaluate("x == 'b:'")
 
-
+    @pytest.mark.thread_unsafe
     def test_no_sanitize(self):
         try: # Errors on compile() after eval()
             evaluate('import os;', sanitize=False)
@@ -642,11 +709,15 @@ class test_evaluate(TestCase):
         n = np.array([-360, -360, -360, 360, 360, 360], dtype=np.int32)
         out_i = evaluate('a % n')
         assert_equal(out_i, np.mod(a, n))
+        main_i = evaluate('a // n')
+        assert_equal(main_i, a // n)
 
         b = a.astype(np.int64)
         m = n.astype(np.int64)
         out_l = evaluate('b % m')
         assert_equal(out_l, np.mod(b, m))
+        main_l = evaluate('b // m')
+        assert_equal(main_l, a // m)
 
     def test_negative_power_scalar(self):
         # Test for issue #428, where the power is negative and the base is an
@@ -659,20 +730,48 @@ class test_evaluate(TestCase):
         out_l = evaluate('base ** -1.0')
         assert_equal(out_l, np.power(base, -1.0))
 
-
     def test_ex_uses_vml(self):
         vml_funcs = [ "sin", "cos", "tan", "arcsin", "arccos", "arctan",
                       "sinh", "cosh", "tanh", "arcsinh", "arccosh", "arctanh",
-                      "log", "log1p","log10", "exp", "expm1", "abs", "conj",
-                      "arctan2", "fmod"]
+                      "log", "log1p","log10", "log2", "exp", "expm1", "abs", "conj",
+                      "arctan2", "fmod", "hypot"]
         for func in vml_funcs:
             strexpr = func+'(a)'
             _, ex_uses_vml = numexpr.necompiler.getExprNames(strexpr, {})
             assert_equal(ex_uses_vml, use_vml, strexpr)
 
+    def test_bool_funcs(self):
+        # Test functions with boolean outputs
+        array_size = 100
+        dtype = np.float32
+        a = np.arange(2 * array_size, dtype=dtype)
+        a[array_size//2] = np.nan
+        a[array_size//3] = np.inf
+        a[array_size//4] = -2
+
+        assert_equal(evaluate("isnan(a)"), isnan(a))
+        assert_equal(evaluate("isfinite(a)"), isfinite(a))
+        assert_equal(evaluate("isinf(a)"), isinf(a))
+        assert_equal(evaluate("signbit(a)"), signbit(a))
+
+        a = a.astype(np.float64)
+        assert a.dtype == np.float64
+        assert_equal(evaluate("isnan(a)"), isnan(a))
+        assert_equal(evaluate("isfinite(a)"), isfinite(a))
+        assert_equal(evaluate("isinf(a)"), isinf(a))
+        assert_equal(evaluate("signbit(a)"), signbit(a))
+
+        a = a.astype(np.complex128)
+        assert a.dtype == np.complex128
+        assert np.all(evaluate("isnan(a)") == np.isnan(a))
+        assert np.all(evaluate("isfinite(a)") == np.isfinite(a))
+        assert np.all(evaluate("isinf(a)") == np.isinf(a))
+        # signbit not defined for complex numbers
+
     if 'sparc' not in platform.machine():
         # Execution order set here so as to not use too many threads
         # during the rest of the execution.  See #33 for details.
+        @pytest.mark.thread_unsafe
         def test_changing_nthreads_00_inc(self):
             a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
@@ -681,6 +780,7 @@ class test_evaluate(TestCase):
                 c = evaluate("((.25*a + .75)*a - 1.5)*a - 2")
                 assert_array_almost_equal(b, c)
 
+        @pytest.mark.thread_unsafe
         def test_changing_nthreads_01_dec(self):
             a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
@@ -733,13 +833,13 @@ func1tests = []
 for func in ['copy', 'ones_like', 'sqrt',
              'sin', 'cos', 'tan', 'arcsin', 'arccos', 'arctan',
              'sinh', 'cosh', 'tanh', 'arcsinh', 'arccosh', 'arctanh',
-             'log', 'log1p', 'log10', 'exp', 'expm1', 'abs', 'conj',
-             'ceil', 'floor']:
+             'log', 'log1p', 'log10', "log2", 'exp', 'expm1', 'abs', 'conj',
+             'ceil', 'floor', 'round', 'trunc', 'sign']:
     func1tests.append("a + %s(b+c)" % func)
 tests.append(('1_ARG_FUNCS', func1tests))
 
 func2tests = []
-for func in ['arctan2', 'fmod']:
+for func in ['arctan2', 'fmod', 'hypot', 'nextafter', 'copysign']:
     func2tests.append("a + %s(b+c, d+1)" % func)
     func2tests.append("a + %s(b+c, 1)" % func)
     func2tests.append("a + %s(1, d+1)" % func)
@@ -782,109 +882,93 @@ def equal(a, b, exact):
 class Skip(Exception): pass
 
 
-def test_expressions():
-    test_no = [0]
-
-    def make_test_method(a, a2, b, c, d, e, x, expr,
-                         test_scalar, dtype, optimization, exact, section):
-        this_locals = locals()
-
-        def method():
-            try:
-                # We don't want to listen at RuntimeWarnings like
-                # "overflows" or "divide by zero" in plain eval().
-                warnings.simplefilter("ignore")
-                npval = eval(expr, globals(), this_locals)
-                warnings.simplefilter("always")
-                npval = eval(expr, globals(), this_locals)
-            except Exception as ex:
-                # just store the exception in a variable
-                # compatibility with numpy v1.12
-                # see also https://github.com/pydata/numexpr/issues/239
-                np_exception = ex
-                npval = None
-            else:
-                np_exception = None
-
-            try:
-                neval = evaluate(expr, local_dict=this_locals,
-                                 optimization=optimization)
-            except AssertionError:
-                raise
-            except NotImplementedError:
-                print('%r not implemented for %s (scalar=%d, opt=%s)'
-                      % (expr, dtype.__name__, test_scalar, optimization))
-            except Exception as ne_exception:
-                same_exc_type = issubclass(type(ne_exception),
-                                           type(np_exception))
-                if np_exception is None or not same_exc_type:
-                    print('numexpr error for expression %r' % (expr,))
-                    raise
-            except:
-                print('numexpr error for expression %r' % (expr,))
-                raise
-            else:
-                msg = ('expected numexpr error not raised for expression '
-                       '%r' % (expr,))
-                assert np_exception is None, msg
-
-                assert equal(npval, neval, exact), """%r
-(test_scalar=%r, dtype=%r, optimization=%r, exact=%r,
- npval=%r (%r - %r)\n neval=%r (%r - %r))""" % (expr, test_scalar, dtype.__name__,
-                                                optimization, exact,
-                                                npval, type(npval), shape(npval),
-                                                neval, type(neval), shape(neval))
-
-        method.description = ('test_expressions(%s, test_scalar=%r, '
-                              'dtype=%r, optimization=%r, exact=%r)') % (expr, test_scalar, dtype.__name__, optimization, exact)
-        test_no[0] += 1
-        method.__name__ = 'test_scalar%d_%s_%s_%s_%04d' % (test_scalar,
-                                                           dtype.__name__,
-                                                           optimization.encode('ascii'),
-                                                           section.encode('ascii'),
-                                                           test_no[0])
-        return method
-
+@pytest.mark.parametrize(
+    "expr,test_scalar,dtype,optimization,exact,section_name",
+    [
+        (expr, test_scalar, dtype, optimization, exact, section_name)
+        for test_scalar in (0, 1, 2)
+        for dtype in (int, int, np.float32, double, complex)
+        for optimization, exact in [
+            ("none", False),
+            ("moderate", False),
+            ("aggressive", False),
+        ]
+        for section_name, section_tests in tests
+        for expr in section_tests
+        if not (
+            dtype == complex
+            and (
+                "<" in expr
+                or ">" in expr
+                or "%" in expr
+                or "arctan2" in expr
+                or "fmod" in expr
+                or "hypot" in expr
+                or "nextafter" in expr
+                or "copysign" in expr
+                or "trunc" in expr
+                or "floor" in expr
+                or "ceil" in expr
+            )
+        )
+        if not (dtype in (int, int) and test_scalar and expr == "(a+1) ** -1")
+    ],
+)
+def test_expressions(
+    expr, test_scalar, dtype, optimization, exact, section_name
+):
+    array_size = 100
+    a = arange(2 * array_size, dtype=dtype)[::2]
+    a2 = zeros([array_size, array_size], dtype=dtype)
+    b = arange(array_size, dtype=dtype) / array_size
+    c = arange(array_size, dtype=dtype)
+    d = arange(array_size, dtype=dtype)
+    e = arange(array_size, dtype=dtype)
     x = None
-    for test_scalar in (0, 1, 2):
-        for dtype in (int, int, np.float32, double, complex):
-            array_size = 100
-            a = arange(2 * array_size, dtype=dtype)[::2]
-            a2 = zeros([array_size, array_size], dtype=dtype)
-            b = arange(array_size, dtype=dtype) / array_size
-            c = arange(array_size, dtype=dtype)
-            d = arange(array_size, dtype=dtype)
-            e = arange(array_size, dtype=dtype)
-            if dtype == complex:
-                a = a.real
-                for x in [a2, b, c, d, e]:
-                    x += 1j
-                    x *= 1 + 1j
-            if test_scalar == 1:
-                a = a[array_size // 2]
-            if test_scalar == 2:
-                b = b[array_size // 2]
-            for optimization, exact in [
-                ('none', False), ('moderate', False), ('aggressive', False)]:
-                for section_name, section_tests in tests:
-                    for expr in section_tests:
-                        if (dtype == complex and
-                            ('<' in expr or '>' in expr or '%' in expr
-                             or "arctan2" in expr or "fmod" in expr
-                             or "floor" in expr or "ceil" in expr)):
-                            # skip complex comparisons or functions not
-                            # defined in complex domain.
-                            continue
-                        if (dtype in (int, int) and test_scalar and
-                                    expr == '(a+1) ** -1'):
-                            continue
 
-                        m = make_test_method(a, a2, b, c, d, e, x,
-                                             expr, test_scalar, dtype,
-                                             optimization, exact,
-                                             section_name)
-                        yield m
+    if dtype == complex:
+        a = a.real
+        for var in [a2, b, c, d, e]:
+            var += 1j
+            var *= 1 + 1j
 
+    if test_scalar == 1:
+        a = a[array_size // 2]
+    if test_scalar == 2:
+        b = b[array_size // 2]
+
+    # We don't want to listen at RuntimeWarnings like
+    # "overflows" or "divide by zero" in plain eval().
+    warnings.simplefilter("ignore")
+    try:
+        npval = eval(expr, globals(), locals())
+    except Exception as ex:
+        np_exception = ex
+        npval = None
+    else:
+        np_exception = None
+    warnings.simplefilter("always")
+
+    try:
+        neval = evaluate(expr, local_dict=locals(), optimization=optimization)
+    except AssertionError:
+        raise
+    except NotImplementedError:
+        pytest.skip(
+            f"{expr!r} not implemented for {dtype.__name__} (scalar={test_scalar}, opt={optimization})"
+        )
+    except Exception as ne_exception:
+        same_exc_type = issubclass(type(ne_exception), type(np_exception))
+        if np_exception is None or not same_exc_type:
+            pytest.fail(f"numexpr error for expression {expr!r}")
+    else:
+        if np_exception is not None:
+            pytest.fail(f"expected numexpr error not raised for expression {expr!r}")
+
+        assert equal(npval, neval, exact), f"""{expr!r}
+            (test_scalar={test_scalar!r}, dtype={dtype.__name__!r}, optimization={optimization!r}, exact={exact!r},
+            npval={npval!r} ({type(npval)!r} - {shape(npval)!r})
+            neval={neval!r} ({type(neval)!r} - {shape(neval)!r}))"""
 
 class test_int64(TestCase):
     def test_neg(self):
@@ -1119,6 +1203,7 @@ def _environment(key, value):
             del os.environ[key]
 
 # Test cases for the threading configuration
+@pytest.mark.thread_unsafe
 class test_threading_config(TestCase):
     def test_max_threads_unset(self):
         # Has to be done in a subprocess as `importlib.reload` doesn't let us
@@ -1128,7 +1213,7 @@ class test_threading_config(TestCase):
                 "if 'NUMEXPR_MAX_THREADS' in os.environ: os.environ.pop('NUMEXPR_MAX_THREADS')",
                 "if 'OMP_NUM_THREADS' in os.environ: os.environ.pop('OMP_NUM_THREADS')",
                 "import numexpr",
-                "assert(numexpr.nthreads <= 8)",
+                f"assert(numexpr.nthreads <= {MAX_THREADS})",
                 "exit(0)"])
         subprocess.check_call([sys.executable, '-c', script])
 
@@ -1164,14 +1249,14 @@ class test_threading_config(TestCase):
             if 'sparc' in platform.machine():
                 self.assertEqual(1, numexpr._init_num_threads())
             else:
-                self.assertEqual(detect_number_of_cores(), numexpr._init_num_threads())
+                self.assertEqual(min(detect_number_of_cores(), MAX_THREADS), numexpr._init_num_threads())
 
     def test_numexpr_max_threads_empty_string(self):
         with _environment('NUMEXPR_MAX_THREADS', ''):
             if 'sparc' in platform.machine():
                 self.assertEqual(1, numexpr._init_num_threads())
             else:
-                self.assertEqual(detect_number_of_cores(), numexpr._init_num_threads())
+                self.assertEqual(min(detect_number_of_cores(), MAX_THREADS), numexpr._init_num_threads())
 
     def test_vml_threads_round_trip(self):
         n_threads = 3
@@ -1200,6 +1285,7 @@ class test_threading(TestCase):
         test.join()
 
     def test_multithread(self):
+
         import threading
 
         # Running evaluate() from multiple threads shouldn't crash
@@ -1217,6 +1303,77 @@ class test_threading(TestCase):
         for t in threads:
             t.join()
 
+    def test_thread_safety(self):
+        """
+        Expected output
+
+        When not safe (before the pr this test is commited)
+        AssertionError: Thread-0 failed: result does not match expected
+
+        When safe (after the pr this test is commited)
+        Should pass without failure
+        """
+        import threading
+        import time
+
+        barrier = threading.Barrier(4)
+
+        # Function that each thread will run with different expressions
+        def thread_function(a_value, b_value, expression, expected_result, results, index):
+            validate(expression, local_dict={"a": a_value, "b": b_value})
+            # Wait for all threads to reach this point
+            # such that they all set _numexpr_last
+            barrier.wait()
+
+            # Simulate some work or a context switch delay
+            time.sleep(0.1)
+
+            result = re_evaluate(local_dict={"a": a_value, "b": b_value})
+            results[index] = np.array_equal(result, expected_result)
+
+        def test_thread_safety_with_numexpr():
+            num_threads = 4
+            array_size = 1000000
+
+            expressions = [
+                "a + b",
+                "a - b",
+                "a * b",
+                "a / b"
+            ]
+
+            a_value = [np.full(array_size, i + 1) for i in range(num_threads)]
+            b_value = [np.full(array_size, (i + 1) * 2) for i in range(num_threads)]
+
+            expected_results = [
+                a_value[i] + b_value[i] if expr == "a + b" else
+                a_value[i] - b_value[i] if expr == "a - b" else
+                a_value[i] * b_value[i] if expr == "a * b" else
+                a_value[i] / b_value[i] if expr == "a / b" else None
+                for i, expr in enumerate(expressions)
+            ]
+
+            results = [None] * num_threads
+            threads = []
+
+            # Create and start threads with different expressions
+            for i in range(num_threads):
+                thread = threading.Thread(
+                    target=thread_function,
+                    args=(a_value[i], b_value[i], expressions[i], expected_results[i], results, i)
+                )
+                threads.append(thread)
+                thread.start()
+
+            for thread in threads:
+                thread.join()
+
+            for i in range(num_threads):
+                if not results[i]:
+                    self.fail(f"Thread-{i} failed: result does not match expected")
+
+        test_thread_safety_with_numexpr()
+
 
 # The worker function for the subprocess (needs to be here because Windows
 # has problems pickling nested functions with the multiprocess module :-/)
@@ -1230,6 +1387,7 @@ def _worker(qout=None):
 
 # Case test for subprocesses (via multiprocessing module)
 class test_subprocess(TestCase):
+    @pytest.mark.thread_unsafe
     def test_multiprocess(self):
         try:
             import multiprocessing as mp
@@ -1252,9 +1410,10 @@ class test_subprocess(TestCase):
 def print_versions():
     """Print the versions of software that numexpr relies on."""
     # from pkg_resources import parse_version
-    from numexpr.cpuinfo import cpu
     import platform
 
+    from numexpr.cpuinfo import cpu
+
     print('-=' * 38)
     print('Numexpr version:   %s' % numexpr.__version__)
     print('NumPy version:     %s' % np.__version__)
@@ -1295,48 +1454,68 @@ test.__test__ = False
 
 
 def suite():
-    import unittest
     import platform as pl
+    import unittest
 
     theSuite = unittest.TestSuite()
     niter = 1
 
-    class TestExpressions(TestCase):
-        pass
+    # Add the pytest parametrized tests only if pytest is available
+    if pytest_available:
+        # Create a class that will run the test_expressions function with different parameters
+        class TestExpressions(unittest.TestCase):
+            pass
 
-    def add_method(func):
-        def method(self):
-            return func()
+        # Get the parameters from the pytest.mark.parametrize decorator
+        # This is safer than accessing internal pytest modules
+        marker = getattr(test_expressions, "pytestmark", None)
+        if marker and hasattr(marker[0], "args") and len(marker[0].args) >= 2:
+            param_list = marker[0].args[1]
 
-        setattr(TestExpressions, func.__name__,
-                method.__get__(None, TestExpressions))
+            # Create test methods dynamically
+            for i, params in enumerate(param_list):
+                expr, test_scalar, dtype, optimization, exact, section_name = params
 
-    for func in test_expressions():
-        add_method(func)
+                def create_test_method(params=params):
+                    def test_method(self):
+                        expr, test_scalar, dtype, optimization, exact, section_name = (
+                            params
+                        )
+                        test_expressions(
+                            expr, test_scalar, dtype, optimization, exact, section_name
+                        )
+
+                    return test_method
+
+                method_name = f"test_expr_{i}"
+                setattr(TestExpressions, method_name, create_test_method())
 
     for n in range(niter):
-        theSuite.addTest(unittest.makeSuite(test_numexpr))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_numexpr))
         if 'sparc' not in platform.machine():
-            theSuite.addTest(unittest.makeSuite(test_numexpr2))
-        theSuite.addTest(unittest.makeSuite(test_evaluate))
-        theSuite.addTest(unittest.makeSuite(TestExpressions))
-        theSuite.addTest(unittest.makeSuite(test_int32_int64))
-        theSuite.addTest(unittest.makeSuite(test_uint32_int64))
-        theSuite.addTest(unittest.makeSuite(test_strings))
+            theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_numexpr2))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_evaluate))
+        # Add the dynamically created TestExpressions to the suite
+        if pytest_available:
+            theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestExpressions))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_int32_int64))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_uint32_int64))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_strings))
         theSuite.addTest(
-            unittest.makeSuite(test_irregular_stride))
-        theSuite.addTest(unittest.makeSuite(test_zerodim))
-        theSuite.addTest(unittest.makeSuite(test_threading_config))
+            unittest.defaultTestLoader.loadTestsFromTestCase(test_irregular_stride))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_zerodim))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_threading_config))
 
         # multiprocessing module is not supported on Hurd/kFreeBSD
         if (pl.system().lower() not in ('gnu', 'gnu/kfreebsd')):
-            theSuite.addTest(unittest.makeSuite(test_subprocess))
+            theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_subprocess))
 
         # I need to put this test after test_subprocess because
         # if not, the test suite locks immediately before test_subproces.
         # This only happens with Windows, so I suspect of a subtle bad
         # interaction with threads and subprocess :-/
-        theSuite.addTest(unittest.makeSuite(test_threading))
+        theSuite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(test_threading))
+
 
     return theSuite
 
